/******************************************************************************
* DSGE_STSTPMOM.G
* computes posterior mean and std and CI based on output of posterior simulator
* steady states
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include dsge_spec.g;
#include path_spec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun   = "1";
nburn   = 1;    /* initial draws to be discarded */
hpdprob = 0.90 ;
taxind  = 4;

if taxind == 0;
   tax_str = "";
elseif taxind == 2;
   tax_str = "tauh022tauk035chi036pct";
elseif taxind == 3;
   tax_str = "tauh029tauk047chi036pct";
elseif taxind == 4;
   tax_str = "tauh029tauk035chi050pct";
elseif taxind == 5;
   tax_str = "tauh0229tauk0443chi0224pct";
elseif taxind == 6;
   tax_str = "tauh0269tauk0327chi0417pct";
endif;

lpath  = resupath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

ldraw  = lpath $+ "\\welfareperturb" $+ tax_str ;
odraws = opath $+ "\\welfareperturbs" $+ tax_str ;      

open fhdraw = ^ldraw for read;

drawrow = readr(fhdraw, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw, nburn);

/********************************************************************
* Part 1: Compute the mean of x(i) and x^2(i)
********************************************************************/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
nblock  = 1000;
ndraws  = 0;

if _verbose;
   "";
   "=============================================";
   "Steady States                                ";
   "";
   "Model            " $lmodel;
   "Data set         " $dataselstr;
   "Prior            " $lprior;
   "Subsample        " $subTstr;
   "MHrun Index      " $mhrun;
   "Coverage         " hpdprob;
   "=============================================";
   "";
   "Part 1: Mean and Standard Deviation";
endif;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);

   drawmean   = drawmean   + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawcross  = drawcross  + drawblock'*drawblock;

   ndraws = ndraws + rows(drawblock);

   if _verbose;
      ".";;pause(1e-20);
   endif;

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);
drawcov    = drawcross/ndraws - drawmean'*drawmean;

/* save steady states: capital, hours, consumption, output, rsmall, wage bill, 
** capital rental, government spending
*/

if taxind == 0;
   draw_names = { Wlf }; 
else;
   draw_names = { Wlf } ; 
endif;

if _verbose;

   /** Report Posterior Mean and stderror **/
   "";
   outmat = ("Parameter"  $~ "Mean" $~ "Stdd") $|
            ( (draw_names') $~ ftocv( (drawmean' ~ real(drawstdd)'),4,4) );

   $outmat;

endif;


/********************************************************************
* Part 2: HPD Interval
********************************************************************/
drawrow = seekr( fhdraw, nburn);
drawci  = zeros( 2,drawdim);

if _verbose;
   "";
   "Part 2: HPD Interval";
endif;

j = 1;
do until j > drawdim;

   /** Read only the j'th column **/
   drawrow   = seekr(fhdraw,nburn);
   drawblock = readr(fhdraw,1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;

   do until eofloop;

      drawblock = readr(fhdraw,nblock);
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);

   endo;

   if _verbose;".";;pause(1e-20);endif;

   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;

/** Report **/
if _verbose;
   "";"";

   outmat = ("Parameter"  $~ "Mean" $~ "Stdd" $~ "CI(LOW)" $~ "CI(HIGH)") $|
            ( (draw_names') $~ ftocv( (drawmean' ~ real(drawstdd)' ~ drawci'),4,2) );

   $outmat;

   "";
endif;

create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws;

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim = cols(draws);
ndraws  = rows(draws);
hpdband = zeros(2,drawdim);
nwidth  = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;

retp(hpdband);
endp;
