/******************************************************************/
/* filename :    vm_dummy.g                                       */
/* Description:  Generates dummy observations                     */
/*               for a Minnesota Prior                            */
/******************************************************************/


new;
closeall; 
library pgraph;
cls;

#include vm_spec.g;
#include path_spec.g;

/******************************************************** 
* Import data series                                    *
********************************************************/

#include loaddata.g;


/** dimensions: YY contains the estimation sample and observations
**  to initialize lags
**/ 
nv      =   cols(YY);         /*number of variables*/ 
nobs    =   rows(YY)-nlags_;  /* number of observations */

/** number of total dummy observations: 
    we need nlags_*nv dummy obs in order to get 
    the priors for the coeficients  at every lag
    PLUS nv (=nv*w, where w=1) dummy obs to 
    get the priors for the elements 
    in the covariance matrix of error terms, 
    PLUS 1*nv dummy obs for the priors of the 
    covariance matrix of coefficients of different lags, 
    PLUS 1 dummy obs to get the priors for the intercept
**/

nd      =   1+(nlags_+w+1)*(nv);

ydum    =   zeros(nd,nv); 
xdum    =   zeros(nd,1+nlags_*nv); 

/** Obtain mean and standard deviation 
**  from expandend pre-sample data
**/ 
YY0     =   YY[1:nlags_+16,.];  
ybar    =   meanc(YY0);      
ybar    =   ybar'; 
sbar    =   stdc(YY0); 
sbar    =   sbar';        
sprior  =   ybar|sbar;   /*2xnvar*/

/** Generate Matrices with Dummy Observations
**/ 

mprior  =   tau|d|w|lambda|mu; 
{xdum,ydum,breakss}=varprior(nv,nlags_,mprior,sprior); 

X0      =   xdum; 
Y0      =   ydum;  

/** Generate Matrices with Actual Observations
**/ 

Y1      =   YY[nlags_+1:nlags_+nobs,.]; 
X1      =   zeros(nobs,nv*nlags_); 

i = 1;
do until i > nlags_;
   X1[.,(i-1)*nv+1:i*nv] = YY[nlags_-(i-1):nlags_+nobs-i,.];
   i = i+1;
endo;

/** the constant is added to the last column of X1. 
**/
X1  =  x1~ones(nobs,1);  

/** Store all observations on hard drive
**/
odumobs = priopath $+ "\\dumobs" $+ ".dat";
create odumobs = ^odumobs with x, nv+nv*nlags_+1,8;  
wr = writer(odumobs, ( (Y0~X0) | (Y1~X1)) );
closeall dumobs;

end;



/***************************************************************
The procedure for generating dummy observations  

****************************************************************/ 
proc(3)=varprior(a,b,c,e); 
/*a:nv, b:nlags_, c:hyperparameters, e:pre-sample moments */ 
local ta, dd, ww, lam, mm,xdu,ydu,breakss,dsize,j,sig,lammean,mumean, lamall, muall;   
/*ta=tau; dd=d; ww=w; lam=lambda; m=mu, e=sprior=ybar|sbar*/
ta=c[1,.];  dd=c[2,.]; ww=c[3,.]; lam=c[4,.]; mm=c[5,.]; 
dsize   =   1+(b+ww+1)*a;             /*number of dummy observations*/
breakss =   zeros(5,1); 

ydu     =   zeros(dsize,a); 
xdu     =   zeros(dsize,1+a*b); 

/*Dummies for the coefficients of the first lag*/ 
sig         =   e[2,.]'; 
sig         =   diagrv(zeros(a,a),sig);             /*it creates a diagonal matrix of dimension axa with sig in the main diagonal*/ 
ydu[1:a,.]  =   ta*sig; 
xdu[1:a,.]  =   (ta*sig)~zeros(a,(b-1)*a+1);        /*4x17 if nv=4 and nlags_=4*/
breakss[1,.]=   a; 

/*Dummies for the coefficients of the remaining lag(we are stacking all of them)*/ 
if b > 1;
   ydu[breakss[1,.]+1:a*b,.]   =   zeros((b-1)*breakss[1,.],a);
   j = 1;
   do until j>b-1; 
     xdu[breakss[1,.]+(j-1)*a+1:breakss[1,.]+j*a,.]     =   zeros(a,j*a)~ta*sig*((j+1)^dd)~zeros(a,1+(b-1-j)*a);  
     j = j+1; 
   endo; 
   breakss[2,.]=breakss[1,.]+(b-1)*a; 
else;
   breakss[2,.]=breakss[1,.]; 
endif;

/*Dummies for the covariance matrix of error terms */ 
ydu[breakss[2,.]+1:breakss[2,.]+ww*a,.] =   ones(ww,1).*.sig; 
breakss[3,.]    =   breakss[2,.]+ww*a; 

/*Dummies for the coefficients of the constant term*/ 

lammean                 =   lam*e[1,.];
ydu[breakss[3,.]+1,.]   =   lammean; 

lamall                  =   lammean;
j = 2;
do until j > nlags_;
   lamall = lamall ~ lammean;
   j = j+1;
endo;
lamall = lamall ~ lam;

xdu[breakss[3,.]+1,.]   =   lamall; 
breakss[4,.]            =   breakss[3,.]+1; 

/*Dummies for the covariance matrix of coefficients of different lags */ 

mumean                              =   diagrv(zeros(a,a),(mm*e[1,.])');
ydu[breakss[4,.]+1:breakss[4,.]+a,.]=   mumean; 

muall                  =   mumean;
j = 2;
do until j > nlags_;
   muall = muall ~ mumean;
   j = j+1;
endo;
muall = muall ~ zeros(a,1);

xdu[breakss[4,.]+1:breakss[4,.]+a,.]=   muall; 
breakss[5,.]                        =   breakss[4,.]+a; 

retp(xdu,ydu,breakss); 
endp; 
